/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Contributor(s): Billy <lamekisser@yahoo.co.uk> http://linkvisitor.mozdev.org/
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


var LinkVisitor = {
  _globalHistory : Components.classes['@mozilla.org/browser/nav-history-service;1'].getService(Components.interfaces.nsIGlobalHistory2),
  _browserHistory : Components.classes["@mozilla.org/browser/nav-history-service;1"].getService(Components.interfaces.nsIBrowserHistory),
  _URIFixup : Components.classes["@mozilla.org/docshell/urifixup;1"].getService(Components.interfaces.nsIURIFixup),
  _bookmarksService : Components.classes['@mozilla.org/browser/nav-bookmarks-service;1'].getService(Components.interfaces.nsINavBookmarksService),
  _pref : Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch('linkvisitor.'),
  
  // This is the node user right clicks on, null if it is not an address node.
  // It is initiliased and reassigned in _isOverLink() only.
  _overLink : null,

  // Place holders for methods. They will be initialised by testBrowserVersion().
  // These are required because different builds use different codes!
  _addPageURI : null,
  _removePageURI : null,
  
   // can we call removePages on the browserHistory?  (new to Firefox 3.1)
  _canRemovePages : false,
  
  // Modified from optimoz mosgestoverlay.js
  // Given a node, try to get the address node of itself or its parent(s).
  // Returns null if cannot find any.
  _findNodeLink : function(domPosition) {
    if (domPosition == document)
      return null;
    try {
      if(domPosition.nodeName == "A")
        return {node : domPosition, uri : domPosition.href};
      else
        return this._findNodeLink(domPosition.parentNode);
    }
    catch(e) {
      return null;
    }
  },

  // Returns true if the popup node is an address node.
  _isOverLink : function() {
    this._overLink = this._findNodeLink(document.popupNode);
    return (this._overLink != null);
  },

  // Idea taken from Tabbrowser Extensions globaloverlay.js
  // Once we added/removed the url to the history list,
  // doing this will change the address node's colour.
  // Unfortunately, it will only affect the node, and not other
  // nodes with the same url in the page.
  _refreshLinkColor : function(aNode) {
    var oldHref = aNode.href;
    aNode.href = '';
    aNode.href = oldHref;
  },

  // This function is invoked in window (?) context,
  // so use 'LinkVisitor' instead of 'this'.
  // event.originalTarget is the loaded document.
  onWindowLoad : function(event) {
    var parentPopup = document.getElementById('contentAreaContextMenu');
    parentPopup.addEventListener("popupshown", LinkVisitor._onParentPopupShown, true);

    if (LinkVisitor._isBookmarkShownVisited())
      LinkVisitor._markBookmarkedLinksAsVisited(event.originalTarget);
  },


  _setupAccelerator : function( id, keys ) {
    var values = keys.split('+');
    if ( values.length <= 1 )   values = keys.split(' ');

    var mods = '';
    var keycode = '';

    for ( var i in values ) {
      if ( values[i] == 'shift' || values[i] == 'alt' || values[i] == 'meta' || values[i] == 'control'  || values[i] == 'accel' ) {
        mods += values[i] + ' ';
      }
      else {
        keycode = values[i];
      }
    } 

    var keyNode = document.getElementById( id );
    keyNode.setAttribute('modifiers', mods );
    keyNode.setAttribute('keycode', keycode );
  },

  setupAccelerators : function() {
    if ( this._pref.getBoolPref('override_default_keys')) {
      try {
	this._setupAccelerator( 'key_vlSelVisitAll',   this._pref.getCharPref( 'keys.selVisitAll' ));
      }
      catch(e) {}
      try {
	this._setupAccelerator( 'key_vlSelUnvisitAll', this._pref.getCharPref( 'keys.selUnvisitAll' ));
      }
      catch(e) {}
      try {
	this._setupAccelerator( 'key_vlVisitAll',      this._pref.getCharPref( 'keys.visitAll' ));
      }
      catch(e) {}
      try {
	this._setupAccelerator( 'key_vlUnvisitAll',    this._pref.getCharPref( 'keys.unvisitAll' ));
      }
      catch(e) {}
    }
  },

  // This function is invoked in window (?) context,
  // so use 'LinkVisitor' instead of 'this'
  _onParentPopupShown : function() {
   if (LinkVisitor._isOverLink()) {
      document.getElementById('vlToggleLink').setAttribute('hidden', false);

      var fixupURI = LinkVisitor._getFixupURI(LinkVisitor._overLink.uri);
      var visited = LinkVisitor._globalHistory.isVisited(fixupURI);
      var bundle = document.getElementById("vlStringBundle");
      var menuItem = document.getElementById('vlToggleLink');
      if (visited) {
        menuItem.label = bundle.getString('vlToggleLinkVisitedLabel');
        menuItem.accesskey = bundle.getString('vlToggleLinkVisitedAccesskey');
      }
      else {
        menuItem.label = bundle.getString('vlToggleLinkUnvisitedLabel');
        menuItem.accesskey = bundle.getString('vlToggleLinkUnvisitedAccesskey');
      }
    }
    else
      document.getElementById('vlToggleLink').setAttribute('hidden', true);
  },

  // If this function is invoked from the context menu,
  // onParentPopupShown will always be invoked first.
  onPopupShown : function() {
    var selectedText = String(this._getSelection());
    var textSelected = selectedText.length > 0;

    var hasLink =
      ("links" in document.commandDispatcher.focusedWindow.document) &&
      (document.commandDispatcher.focusedWindow.document.links.length > 0);
    var linkSelected = textSelected && hasLink;

    document.getElementById('vlVisitAll').setAttribute('disabled', !hasLink);
    document.getElementById('vlUnvisitAll').setAttribute('disabled', !hasLink);
    document.getElementById('vlSelVisitAll').setAttribute('disabled', !linkSelected);
    document.getElementById('vlSelUnvisitAll').setAttribute('disabled', !linkSelected);

    document.getElementById('vlMVisitAll').setAttribute('disabled', !hasLink);
    document.getElementById('vlMUnvisitAll').setAttribute('disabled', !hasLink);
    document.getElementById('vlMSelVisitAll').setAttribute('disabled', !linkSelected);
    document.getElementById('vlMSelUnvisitAll').setAttribute('disabled', !linkSelected);

//    var bookmarkVisited = this._isBookmarkShownVisited();
//    document.getElementById('vlBookmarkVisited').setAttribute('checked', bookmarkVisited);
//    document.getElementById('vlMBookmarkVisited').setAttribute('checked', bookmarkVisited);
  },

  // Modified from linky linky.js
  _getSelection : function() {
    var focusedWindow = document.commandDispatcher.focusedWindow;
    return focusedWindow.getSelection();
  },

  // Do the users want bookmarks always shown visited?
  _isBookmarkShownVisited : function() {
    try {
      return this._pref.getBoolPref('always_mark_bookmarks_visited');
    }
    catch(e) {
      return false;
    }
  },

  // Returns true if the url is bookmarked.
  // It seems that nsiBookmarksService uses nsiFixupURI to
  // fix the url, but to be safe, call _getFixupURI and use fixupURI.spec to
  // fix the url ourselves first before calling this function.
  _isBookmarked : function(url) {
    try {
      var found = this._bookmarksService.isBookmarked(url);
      return found;
    }
    catch(e) {
      return false;
    }
  },

  toggleLink : function() {
    if (this._overLink == null)
      return;

    this._testBrowserVersion();

    var fixupURI = this._getFixupURI(this._overLink.uri);
    var visited = this._globalHistory.isVisited(fixupURI);
    //  alert('visited: ' + visited);
    if (visited) {
      if (this._isBookmarkShownVisited() && this._isBookmarked(fixupURI))
        alert(document.getElementById('vlStringBundle').getString('vlBookmarkVisited'));
      else
try {
        this._removePageURI(fixupURI);
}catch(e) { alert(  e ); }
    }
    else
      this._addPageURI(fixupURI, this._getCurrentURI());

    this._refreshLinkColor(this._overLink.node);
  },

  // Return current page URI, may be used as the referrer.
  _getCurrentURI : function() {
    // getWebNavigation() is document.getElementById("content") in browser.js
    return getWebNavigation().currentURI;
  },

  _getFixupURI : function(uri) {
      var fixedURI = this._URIFixup.createFixupURI(uri, 0);
      return fixedURI;
  },

  // Modified from linky linky.js
  markSelectedLinks : function(wantVisited) {
    var selection = this._getSelection();

    if (String(selection).length == 0 ||
        !("links" in document.commandDispatcher.focusedWindow.document))
      return;

    var links = document.commandDispatcher.focusedWindow.document.links;
    if (links.length == 0)
      return;

    this._testBrowserVersion();

    var bookmarkVisited = this._isBookmarkShownVisited();
    var bookmarkCount = 0;
    var currentURI = this._getCurrentURI();

    var urisToProcess = new Array( links.length );
    var linksToRefresh = new Array( links.length );
    var numLinks = 0;

    // not very efficient, but it does the work all right.
    for (var i = 0; i < links.length; i++) {
      if (selection.containsNode(links[i], true)) {
        var fixupURI = this._getFixupURI(links[i].href);
        if ( wantVisited ) {
          if ( !this._globalHistory.isVisited(fixupURI)) {
            urisToProcess[numLinks] = fixupURI;
            linksToRefresh[numLinks++] = links[i];
          }
        }
        else if ( this._globalHistory.isVisited(fixupURI)) {
          if (bookmarkVisited && this._isBookmarked(fixupURI))
            bookmarkCount++; // do not remove the link from history
          else {
            urisToProcess[numLinks] = fixupURI;
            linksToRefresh[numLinks++] = links[i];
          }
        }
      }
    }

    if ( numLinks > 0 ) {
      if ( wantVisited ) this._addPages( numLinks, urisToProcess, linksToRefresh );
      else this._removePages( numLinks, urisToProcess, linksToRefresh );
    }

    if (bookmarkVisited && bookmarkCount > 0) {
      var text = document.getElementById('vlStringBundle').getString('vlBookmarkCount');
      text = this._replaceInsert(text, 1, bookmarkCount);
      alert(text);
    }
  },

  markAllLinks : function(wantVisited) {
    if (!("links" in document.commandDispatcher.focusedWindow.document))
      return;

    var links = document.commandDispatcher.focusedWindow.document.links;
    if (links.length == 0)
      return;

    this._testBrowserVersion();

    var bookmarkVisited = this._isBookmarkShownVisited();
    var bookmarkCount = 0;
    var currentURI = this._getCurrentURI();

    var urisToProcess = new Array( links.length );
    var linksToRefresh = new Array( links.length );
    var numLinks = 0;

    for (var i = 0; i < links.length; i++) {
      var fixupURI = this._getFixupURI(links[i].href);
      if ( wantVisited ) {
        if ( !this._globalHistory.isVisited(fixupURI)) {
          urisToProcess[numLinks] = fixupURI;
          linksToRefresh[numLinks++] = links[i];
        }
      }
      else if ( this._globalHistory.isVisited(fixupURI)) {
        if (bookmarkVisited && this._isBookmarked(fixupURI))
        bookmarkCount++; // do not remove the link from history
        else {
          urisToProcess[numLinks] = fixupURI;
          linksToRefresh[numLinks++] = links[i];
        }
      }
    }

    if ( numLinks > 0 ) {
      if ( wantVisited ) this._addPages( numLinks, urisToProcess, linksToRefresh );
      else this._removePages( numLinks, urisToProcess, linksToRefresh );
    }

    if (bookmarkVisited && bookmarkCount > 0) {
      var text = document.getElementById('vlStringBundle').getString('vlBookmarkCount');
      text = this._replaceInsert(text, 1, bookmarkCount);
      alert(text);
    }
  },

  // Traverse all links in document, and add the links to
  // history if they are not visited but bookmarked.
  _markBookmarkedLinksAsVisited : function(doc) {
    if (!("links" in doc))
      return;

    var links = doc.links;
    if (links.length == 0)
      return;

    this._testBrowserVersion();

    var currentURI = this._getCurrentURI();

    for (var i = 0; i < links.length; i++) {
      var fixupURI = this._getFixupURI(links[i].href);
      if (!this._globalHistory.isVisited(fixupURI) && this._isBookmarked(fixupURI)) {
        this._addPageURI(fixupURI, currentURI);
        this._refreshLinkColor(links[i]);
      }
    }
  },
  
  _numToProcess: 0,
  _lastProcessed: 0,
  _urisToProcess: null,
  _linksToProcess: null,
  _refererURI: null,
  _removePageTask : function( ) {
    if ( LinkVisitor._lastProcessed < LinkVisitor._numToProcess ) { 
      LinkVisitor._removePageURI( LinkVisitor._urisToProcess[LinkVisitor._lastProcessed] );
      try {
        LinkVisitor._refreshLinkColor( LinkVisitor._linksToProcess[LinkVisitor._lastProcessed++] );
      } 
      catch(e) {}
      window.setTimeout( LinkVisitor._removePageTask, 50 );
    }
    else
    {
      LinkVisitor._numToProcess = LinkVisitor._lastProcessed = 0;
      LinkVisitor._urisToProcess = null;
      LinkVisitor._linksToProcess = null;
    }
  },
  
  _removePages: function( numLinks, uris, links ) {
    if ( this._canRemovePages ) {
        this._browserHistory.removePages( uris, numLinks, false );
        for( var i = 0; i < numLinks;  this._refreshLinkColor( links[i++] ));
    }
    else {
      this._numToProcess = numLinks;
      this._lastProcessed = 0;
      this._urisToProcess = uris;
      this._linksToProcess = links;
      window.setTimeout( LinkVisitor._removePageTask, 50 );
    }
  },

  _addPageTask : function( ) {
    if ( LinkVisitor._lastProcessed < LinkVisitor._numToProcess ) { 
      var i = LinkVisitor._numToProcess - 10;
      if ( i < 0 ) i = 0;
      while( LinkVisitor._numToProcess > i ) {
        LinkVisitor._addPageURI( LinkVisitor._urisToProcess[LinkVisitor._lastProcessed] , LinkVisitor._refererURI );
        try {
          LinkVisitor._refreshLinkColor( LinkVisitor._linksToProcess[LinkVisitor._lastProcessed++] );
        } 
        catch(e) {}
      }
      window.setTimeout( LinkVisitor._addPageTask, 50 );
    }
    else
    {
      LinkVisitor._numToProcess = LinkVisitor._lastProcessed = 0;
      LinkVisitor._urisToProcess = null;
      LinkVisitor._linksToProcess = null;
      LinkVisitor._refererURI = null;
    }
  },

  _addPages: function( numLinks, uris, links ) {
    this._numToProcess = numLinks;
    this._lastProcessed = 0;
    this._urisToProcess = uris;
    this._linksToProcess = links;
    this._refererURI = this._getCurrentURI();
    window.setTimeout( LinkVisitor._addPageTask, 50 );
  },

  // Seamonkey 1.8a4 20040927
  _addPageURIWithReferrer: function(pageURI, referrerURI) {
    this._globalHistory.addURI(pageURI, false, true, referrerURI);
  },

  // Build 20040210
  _addPageURIWithoutReferrer: function(pageURI) {
    this._globalHistory.addURI(pageURI, false, true, null );
  },

  // Firefox 0.10.1 20040911 (hah!)
  _removePageURIUsingURI: function(pageURI) {
    this._browserHistory.removePage(pageURI);
  },

  // Build 20040210 and Seamonkey 1.8a4 20040927 (hah!)
  _removePageURIUsingSpec: function(pageURI) {
    this._browserHistory.removePage(pageURI);
  },

  _doNothing : function () {
  },

  _testBrowserVersion : function () {
    if (this._addPageURI != null || this._removePageURI != null)
      return;

    var testURI = this._getFixupURI('linkvisitor.mozdev.org/test');

    try {
      this._addPageURIWithoutReferrer(testURI);
      this._addPageURI = this._addPageURIWithoutReferrer; // yay!
      //alert('_addPageURIWithoutReferrer');
    }
    catch(e) {alert( e );}
    if (this._addPageURI == null) {
      try {
        this._addPageURIWithReferrer(testURI, testURI);
        this._addPageURI = this._addPageURIWithReferrer; // yay!
        //alert('_addPageURIWithReferrer');
      }
      catch(e) {
        this._addPageURI = this._doNothing; // boo!
        //alert('_doNothing');
      }
    }

    try {
      var uris = new Array( testURI );
      this._browserHistory.removePages( uris, 1, false );
      this._canRemovePages = true;
    }
    catch(e) { alert( e ); }

    try {
      this._removePageURIUsingSpec(testURI);
      this._removePageURI = this._removePageURIUsingSpec; // yay!
        //alert('_removePageURIUsingSpec');
    }
    catch(e) {}
    if (this._removePageURI == null) {
      try {
        this._removePageURIUsingURI(testURI);
        this._removePageURI = this._removePageURIUsingURI; // yay!
        //alert('_removePageURIUsingURI');
      }
      catch(e) {
        this._removePageURI = this._doNothing; // boo!
        //alert('_doNothing');
      }
    }
  },

  // Taken from linky's shared.js
  _replaceInsert : function(text, index, value) {
    var result = text;
    var re = new RegExp("#" + index);
    result = result.replace(re, value);
    return result;
  },
};

window.addEventListener("load",  function () {
    LinkVisitor.setupAccelerators();
    gBrowser.addEventListener("load", LinkVisitor.onWindowLoad, true);
  },
  false
);

